'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var transformAnalyzerResult = require('./chunk-transform-analyzer-result-5ead5a6d.js');
var tsModule = require('typescript');
require('ts-simple-type');
var fs = require('fs');
var path = require('path');

/**
 * Analyzes components in code
 * @param {IVirtualSourceFile[]|VirtualSourceFile} inputFiles
 * @param config
 */
function analyzeText(inputFiles, config) {
    if (config === void 0) { config = {}; }
    var ts = config.ts || tsModule;
    // "sys" can be undefined when running in the browser
    var system = ts.sys;
    // Convert arguments into virtual source files
    var files = (Array.isArray(inputFiles) ? inputFiles : [inputFiles])
        .map(function (file) {
        return typeof file === "string"
            ? {
                text: file,
                fileName: "auto-generated-" + Math.floor(Math.random() * 100000) + ".ts",
                entry: true
            }
            : file;
    })
        .map(function (file) { return (transformAnalyzerResult.__assign(transformAnalyzerResult.__assign({}, file), { fileName: file.fileName })); });
    var includeLib = files.some(function (file) { return file.includeLib; });
    var readFile = function (fileName) {
        var matchedFile = files.find(function (currentFile) { return currentFile.fileName === fileName; });
        if (matchedFile != null) {
            return matchedFile.text;
        }
        if (includeLib) {
            // TODO: find better method of finding the current typescript module path
            fileName = fileName.match(/[/\\]/) ? fileName : path.join(path.dirname(require.resolve("typescript")), fileName);
        }
        if (fs.existsSync(fileName)) {
            return fs.readFileSync(fileName, "utf8").toString();
        }
        return undefined;
    };
    var fileExists = function (fileName) {
        return files.some(function (currentFile) { return currentFile.fileName === fileName; });
    };
    var compilerOptions = {
        module: ts.ModuleKind.ESNext,
        target: ts.ScriptTarget.ESNext,
        allowJs: true,
        sourceMap: false,
        strictNullChecks: true
    };
    var program = ts.createProgram({
        rootNames: files.map(function (file) { return file.fileName; }),
        options: compilerOptions,
        host: {
            writeFile: function () { },
            readFile: readFile,
            fileExists: fileExists,
            getSourceFile: function (fileName, languageVersion) {
                var sourceText = this.readFile(fileName);
                if (sourceText == null)
                    return undefined;
                return ts.createSourceFile(fileName, sourceText, languageVersion, true, fileName.endsWith(".js") ? tsModule.ScriptKind.JS : tsModule.ScriptKind.TS);
            },
            getCurrentDirectory: function () {
                return ".";
            },
            getDirectories: function (directoryName) {
                var _a;
                return (_a = system === null || system === void 0 ? void 0 : system.getDirectories(directoryName)) !== null && _a !== void 0 ? _a : [];
            },
            getDefaultLibFileName: function (options) {
                return ts.getDefaultLibFileName(options);
            },
            getCanonicalFileName: function (fileName) {
                return this.useCaseSensitiveFileNames() ? fileName : fileName.toLowerCase();
            },
            getNewLine: function () {
                var _a;
                return (_a = system === null || system === void 0 ? void 0 : system.newLine) !== null && _a !== void 0 ? _a : "\n";
            },
            useCaseSensitiveFileNames: function () {
                var _a;
                return (_a = system === null || system === void 0 ? void 0 : system.useCaseSensitiveFileNames) !== null && _a !== void 0 ? _a : false;
            }
        }
    });
    var checker = program.getTypeChecker();
    // Analyze source files
    var sourceFilesToAnalyze = transformAnalyzerResult.arrayDefined(files.filter(function (file) { return file.analyze !== false; }).map(function (file) { return program.getSourceFile(file.fileName); }));
    var results = sourceFilesToAnalyze.map(function (sf) { return transformAnalyzerResult.analyzeSourceFile(sf, transformAnalyzerResult.__assign({ program: program }, config)); });
    return { checker: checker, program: program, results: results, analyzedSourceFiles: sourceFilesToAnalyze };
}

/**
 * This function only analyzes the HTMLElement declaration found in "lib.dom.d.ts" source file provided by Typescript.
 * @param program
 * @param ts
 */
function analyzeHTMLElement(program, ts) {
    if (ts === void 0) { ts = tsModule; }
    var endsWithLibDom = "lib.dom.d.ts";
    var domLibSourceFile = program.getSourceFiles().find(function (sf) { return sf.fileName.endsWith(endsWithLibDom); });
    if (domLibSourceFile == null) {
        return undefined;
        //throw new Error(`Couldn't find '${endsWith}'. Have you included the 'dom' lib in your tsconfig?`);
    }
    return visit(domLibSourceFile, transformAnalyzerResult.__assign(transformAnalyzerResult.__assign({}, transformAnalyzerResult.makeContextFromConfig({
        program: program,
        ts: ts,
        flavors: [new transformAnalyzerResult.CustomElementFlavor()],
        config: {
            analyzeDefaultLib: true,
            features: transformAnalyzerResult.ALL_COMPONENT_FEATURES
        }
    })), { cache: {
            featureCollection: transformAnalyzerResult.DEFAULT_FEATURE_COLLECTION_CACHE,
            componentDeclarationInSourceFile: transformAnalyzerResult.DEFAULT_COMPONENT_DECLARATION_CACHE,
            general: new Map()
        } }));
}
function visit(node, context) {
    if (context.ts.isInterfaceDeclaration(node) && node.name != null && node.name.text === "HTMLElement") {
        return transformAnalyzerResult.analyzeComponentDeclaration([node], context);
    }
    return node.forEachChild(function (child) {
        return visit(child, context);
    });
}

exports.ALL_COMPONENT_FEATURES = transformAnalyzerResult.ALL_COMPONENT_FEATURES;
exports.JsDocFlavor = transformAnalyzerResult.JsDocFlavor;
exports.VERSION = transformAnalyzerResult.VERSION;
exports.analyzeSourceFile = transformAnalyzerResult.analyzeSourceFile;
exports.getExtendsHeritageClauses = transformAnalyzerResult.getExtendsHeritageClauses;
exports.getExtendsHeritageClausesInChain = transformAnalyzerResult.getExtendsHeritageClausesInChain;
exports.getJsDoc = transformAnalyzerResult.getJsDoc;
exports.getJsDocType = transformAnalyzerResult.getJsDocType;
exports.getMixinHeritageClauses = transformAnalyzerResult.getMixinHeritageClauses;
exports.getMixinHeritageClausesInChain = transformAnalyzerResult.getMixinHeritageClausesInChain;
exports.getSuperclassHeritageClause = transformAnalyzerResult.getSuperclassHeritageClause;
exports.parseSimpleJsDocTypeExpression = transformAnalyzerResult.parseSimpleJsDocTypeExpression;
exports.transformAnalyzerResult = transformAnalyzerResult.transformAnalyzerResult;
exports.visitAllHeritageClauses = transformAnalyzerResult.visitAllHeritageClauses;
exports.analyzeHTMLElement = analyzeHTMLElement;
exports.analyzeText = analyzeText;
